#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>

#define COUNT _BV(PB1)
#define TXD _BV(PB0)
#define BAUD_RATE 9600
#define WINDOW 96000

volatile unsigned char txState;
volatile unsigned char txByte;

volatile unsigned long baudTicks;
volatile unsigned int count;

ISR(TIM0_COMPA_vect) {
	switch(txState) {
		case 0:
		case 1:
			// idle or stop bit
			PORTB |= TXD;
			break;
		case 10:
			// start bit
			PORTB &= (unsigned char)~TXD;
			break;
		default:
			// data bit
			if((txByte >> (9 - txState)) & 0x1) PORTB |= TXD;
			else PORTB &= (unsigned char)~TXD;
			break;
	}
	if(txState) txState--;
	baudTicks++;
}


ISR(PCINT0_vect) {
	if(PINB & COUNT) count++;
}

void sendChar(unsigned char c) {
	while(txState) {
		set_sleep_mode(SLEEP_MODE_IDLE);
		sleep_cpu();	// wait for TX to finish
	}
	txByte = c;
	txState = 10;
}

unsigned long mags[6] PROGMEM = {
	100000,
	10000,
	1000,
	100,
	10,
	1
};

void sendUInt4(unsigned int num) {
	unsigned char i, n, first = 1;
	unsigned long m;
	for(i = 0; i < 6; i++) {
		m = pgm_read_dword(&mags[(unsigned)i]);
		for(n = 0; num >= m; num -= m) n++;
		if(first && n < 1 && i != 5) continue;
		first = 0;
		sendChar('0' + n);
	}
}

void init() {
	count = 0;
	txState = 0;	// idle
	
	// pin change interrupt for counting chamber events
	PORTB |= COUNT;
	PCMSK |= COUNT;
	GIMSK |= _BV(PCIF);
	
	// timer for baud rate
	TCCR0A = _BV(WGM01);		// Clear timer on compare match mode
	TCCR0B = _BV(CS02);		// Clock prescale by 256
	OCR0A = F_CPU/256/BAUD_RATE;	// Compare match at baud-rate
	TIMSK0 = _BV(OCIE0A);		// Enable compare match interrupts

	// TXD pin
	DDRB |= TXD;	// TX pin output enable
	PORTB |= TXD;	// idle

	set_sleep_mode(SLEEP_MODE_IDLE);
	sleep_enable();
	sei();
}

int main() {
	unsigned int countTmp;

	init();

	while(1) {
		if(baudTicks >= WINDOW) {
			cli();
			countTmp = count;
			count = 0;
			baudTicks = 0;
			sei();
			sendUInt4(countTmp);
			sendChar('\r');
			sendChar('\n');
		}
		set_sleep_mode(SLEEP_MODE_IDLE);
		sleep_cpu();
	}

	return 0;  // unreached
}
